/*************************************************************************
 *
 * AIZO CONFIDENTIAL
 * __________________
 *
 *  [2011] - [2012] AIZO AG
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Aizo AG. The intellectual and technical concepts
 * contained herein are proprietary to Aizo AG and may be covered
 * by European and Foreign Patents, patents in process, and are
 * protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from AIZO AG.
 */

Ext.define('DSS.addon.Metering', {

    extend : 'DSS.addon.Framework',

    uses : ['DSS.addon.Metering.MainPanel', 'Ext.Component'],

    config : {
        appName : _('Metering'),
        appId : 'metering',
        appVersion : '1.0.0',
        appIcon : 'images/dss/default_icon.png',
        appLang : 'de_DE'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            loader : {
                url : 'locale/' + this.appLang + '/LC_MESSAGES/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },
    getContent : function() {
        return Ext.create("DSS.addon.Metering.MainPanel", this.config);
    }
});

Ext.onReady(function() {
    var metering = Ext.create('DSS.addon.Metering');
    metering.initPage();
});
/*######################################
 static configs
 ######################################*/

DSS.addon.Metering.time = {
    1 : _('secondly'),
    60 : _('minutely'),
    900 : _('15 minutely'),
    86400 : _('daily'),
    604800 : _('weekly'),
    2592000 : _('monthly')
};

DSS.addon.Metering.lineSteps = {
    1 : [Ext.Date.MINUTE, 1],
    60 : [Ext.Date.HOUR, 1]
};

DSS.addon.Metering.lineMinorTicks = {
    1 : 5,
    60 : 1
};

DSS.addon.Metering.columnSteps = {
    60 : [Ext.Date.MINUTE, 30],
    900 : [Ext.Date.HOUR, 6],
    86400 : [Ext.Date.DAY, 7],
    604800 : [Ext.Date.MONTH, 3],
    2592000 : [Ext.Date.MONTH, 4]
};

DSS.addon.Metering.columnMinorTicks = {
    60 : 1,
    900 : 5,
    86400 : 6,
    604800 : 4,
    2592000 : 8
};

DSS.addon.Metering.dateFormat = {
    1 : 'G:i',
    60 : 'G:i',
    900 : 'j. M - G:i',
    86400 : 'j. M Y',
    604800 : 'j. M Y',
    2592000 : 'j. M Y'
};

DSS.addon.Metering.sceneNames = {
    0 : _("Off"),
    5 : _("Scene 1"),
    17 : _("Scene 2"),
    18 : _("Scene 3"),
    19 : _("Scene 4"),
    1 : _("Area 1 off"),
    6 : _("Area 1 on"),
    2 : _("Area 2 off"),
    7 : _("Area 2 on"),
    3 : _("Area 3 off"),
    8 : _("Area 3 on"),
    4 : _("Area 4 off"),
    9 : _("Area 4 on"),

    32 : _("Off (Scene 10)"),
    33 : _("Scene 11"),
    20 : _("Scene 12"),
    21 : _("Scene 13"),
    22 : _("Scene 14"),

    34 : _("Off (Scene 20)"),
    35 : _("Scene 21"),
    23 : _("Scene 22"),
    24 : _("Scene 23"),
    25 : _("Scene 24"),

    36 : _("Off (Scene 30)"),
    37 : _("Scene 31"),
    26 : _("Scene 32"),
    27 : _("Scene 33"),
    28 : _("Scene 34"),

    38 : _("Off (Scene 40)"),
    39 : _("Scene 41"),
    29 : _("Scene 42"),
    30 : _("Scene 43"),
    31 : _("Scene 44"),

    50 : _("off"),
    51 : _("on"),

    67 : _("Standby"),
    64 : _("Auto Standby"),
    68 : _("Deep off"),
    69 : _("Sleeping"),
    70 : _("Wake up"),
    72 : _("Leave home"),
    71 : _("Coming home"),
    73 : _("Door bell"),
    65 : _("Panic")
};

DSS.addon.Metering.refreshInterval = 2000;

DSS.addon.Metering.YAxisMax = 20;

DSS.addon.Metering.columnValueCount = 150;

/*######################################
 MainPanel
 ######################################*/
Ext.define('DSS.addon.Metering.MainPanel', {
    extend : 'Ext.panel.Panel',

    alias : 'widget.mainpanel',

    initComponent : function() {

        this.dSMPanel = Ext.create('DSS.addon.Metering.dSMPanel');
        this.centerPanel = Ext.create('DSS.addon.Metering.CenterPanel');

        Ext.apply(this, {
            split : true,
            border : true,
            layout : {
                type : 'border'
            },
            items : [this.dSMPanel, this.centerPanel]
        });

        DSS.addon.Metering.MainPanel.superclass.initComponent.apply(this, arguments);
    }
});

/*######################################
 dSMPanel
 ######################################*/
Ext.define('DSS.addon.Metering.dSMPanel', {
    extend : 'Ext.panel.Panel',

    alias : 'widget.dsmpanel',

    initComponent : function() {

        this.dSMView = Ext.create('DSS.addon.Metering.dSMView');

        Ext.apply(this, {
            region : 'west',
            border : false,
            split : true,
            minWidth : 50,
            width : 200,
            layout : {
                type : 'fit'
            },
            items : [this.dSMView]
        });

        DSS.addon.Metering.dSMPanel.superclass.initComponent.apply(this, arguments);
    }
});

/*######################################
 dSMView
 ######################################*/
Ext.define('DSS.addon.Metering.dSMView', {
    extend : 'Ext.view.View',

    initComponent : function() {
        var me = this;
        this.circuitStore = Ext.create('DSS.addon.Metering.stores.CircuitViewStore');
        //TODO: remove the following comment as soon as metering data for the
        // whole installation is available
        /*this.circuitStore.add({
         name : _("whole installation"),
         dsid : 0
         });*/

        this.circuitStore.load();

        var circuitTemplate = new Ext.XTemplate('<tpl for=".">', '<div class="zone-wrap" id="zone-{dsid}">', '<span>', '{text}', '</span>', '</div>', '</tpl>', '<div class="x-clear"></div>');

        Ext.apply(this, {
            title : _("dSM Overview"),
            store : this.circuitStore,
            tpl : circuitTemplate,
            autoScroll : true,
            overflow : 'auto',
            singleSelect : true,
            multiSelect : false,
            simpleSelect : false,
            trackOver : true,
            overItemClass : 'x-item-over',
            overClass : 'x-item-over',
            itemSelector : 'div.zone-wrap',
            emptyText : _("loading dSMs"),
            listeners : {
                selectionchange : function(dv, nodes) {
                    this.getSelectedDSID();
                    this.findParentByType('dsmpanel').findParentByType('mainpanel').centerPanel.repaint();
                }
            }

        });
        this.refresh();

        DSS.addon.Metering.dSMView.superclass.initComponent.apply(this, arguments);
    },
    getSelectedDSID : function() {
        var me = this;

        if(this.getSelectionModel().getCount() === 0) {
            if(this.circuitStore.isLoading()) {
                this.circuitStore.on('load', function() {
                    me.getSelectionModel().select(0);
                    me.findParentByType('dsmpanel').findParentByType('mainpanel').centerPanel.repaint();
                });
                return 0;
            } else {
                this.getSelectionModel().select(0);
            }
        }

        var node = this.getSelectionModel().getSelection();
        if(node !== null) {
            var dsid = node[0].get('dsid');
            if(dsid === 0) {
                //whole apartment is selected -> select next dsm
                this.getSelectionModel().select(1);
                node = this.getSelectionModel().getSelection();
                if(node !== null) {
                    dsid = node[0].get('dsid');
                }
            }
            return dsid;
        } else {
            Ext.MessageBox.alert(_('Error'), _('please select a dSM!'));
        }

        return 0;
    },
    getSelectedName : function() {
        if(this.getSelectionModel().getCount() === 0) {
            this.getSelectionModel().select(0);
        }

        var node = this.getSelectionModel().getSelection();
        if(node !== null) {
            var dsid = node[0].get('text');
            if(dsid === 0) {
                //whole apartment is selected -> select next dsm
                this.getSelectionModel().select(1);
                node = this.getSelectionModel().getSelection();
                if(node !== null) {
                    dsid = node[0].get('text');
                }
            }
            return dsid;
        } else {
            Ext.MessageBox.alert(_('Error'), _('please select a dSM!'));
        }

        return 0;
    }
});

/*######################################
 CenterPanel
 ######################################*/

Ext.define('DSS.addon.Metering.CenterPanel', {
    extend : 'Ext.panel.Panel',

    initComponent : function() {
        var me = this;

        this.unit = '';

        //these two variables are used, if the information about a scene-call is
        // not available at once. Then a counter is started and after a while the
        // information is fetched again.
        this.repaintCount = 0;
        this.repaintCounter = 0;

        this.resolutionStore = Ext.create('DSS.addon.Metering.stores.ResolutionStore');
        this.resolutionStore.load();

        if(this.resolutionStore.isLoading()) {
            this.resolutionStore.on('load', function() {
                me.initStores();
            });
        } else {
            this.initStores();
        }

        this.sceneNameStore = Ext.create('DSS.addon.Metering.stores.SceneNameStore');
        this.sceneNameStore.load();

        //this store is used to display the right events in the realtime charts
        this.apartmentStore = Ext.create('DSS.apartment.ApartmentStore');
        this.apartmentStore.load();

        this.typeStore = Ext.create('Ext.data.Store', {
            fields : ['type', 'name'],
            data : [{
                "type" : "consumption",
                "name" : _('instantaneous consumption') + ' (W)'
            }, {
                "type" : "energy",
                "name" : _('consumption') + ' (Wh)'
            }]
        });

        this.typeBox = Ext.create('Ext.form.field.ComboBox', {
            store : this.typeStore,
            oldValue : '',
            labelAlign : 'top',
            editable : false,
            displayField : 'name',
            valueField : 'type',
            fieldLabel : _('unit'),
            width : 190
        });
        this.typeBox.on('change', function() {
            me.repaint();
        });

        this.timeBox = Ext.create('Ext.form.field.ComboBox', {
            store : this.resolutionStore,
            oldValue : '',
            labelAlign : 'top',
            editable : false,
            queryMode : 'remote',
            lastQuery : '',
            forceSelection : true,
            displayField : 'name',
            valueField : 'resolution',
            fieldLabel : _('resolution'),
            width : 100
        });
        this.timeBox.on('change', function() {
            me.repaint();
        });

        this.zoomSlider = Ext.create('Ext.slider.Slider', {
            width : 100,
            value : 100,
            increment : 10,
            minValue : 100,
            maxValue : 600,
            useTips : false,

            reset : function() {
                me.zoomSlider.setValue(100);
            }
        });

        this.zoomSlider.on('changecomplete', function() {
            var box = me.lineChart.chartBBox;
            me.lineChart.zoomX = Math.min(me.lineChart.zoomX, box.width - box.width / (me.zoomSlider.getValue() / 100));
            me.setChartZoom();
        });

        this.incrementZoomButton = Ext.create('Ext.button.Button', {
            cls : 'x-button-transparent',
            icon : 'images/dss/add.gif',
            handler : function() {
                me.zoomSlider.setValue(me.zoomSlider.getValue() + 100, true);
                me.setChartZoom();
            }
        });

        this.decrementZoomButton = Ext.create('Ext.button.Button', {
            cls : 'x-button-transparent',
            icon : 'images/dss/delete.png',
            handler : function() {
                me.zoomSlider.setValue(me.zoomSlider.getValue() - 100, true);
                me.setChartZoom();
            }
        });

        this.zoomFieldContainer = Ext.create('Ext.form.FieldContainer', {
            fieldLabel : _("Zoom"),
            labelAlign : 'top',
            layout : {
                type : 'table',
                columns : 3
            },
            items : [me.decrementZoomButton, me.zoomSlider, me.incrementZoomButton]
        });

        this.panSlider = Ext.create('Ext.slider.Slider', {
            width : '100',
            value : 0,
            increment : 5,
            minValue : 0,
            maxValue : 100,
            cls : 'x-slider-background',
            useTips : false,

            reset : function() {
                me.panSlider.setValue(0);
            }
        });

        this.panSlider.on('change', function() {
            me.setChartZoom();
        });

        this.exportCSVButton = Ext.create('Ext.button.Button', {
            text : _("export as CSV"),
            iconCls : 'icon-export',
            handler : function() {
                var dSMName = this.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedName();
                var date = Ext.Date.format(new Date(), 'd-m-Y');

                // a temporary form has to be used so that the download
                var tempForm = document.createElement("form");
                tempForm.action = '/cgi-bin/pushfile.hs';
                tempForm.method = "POST";
                tempForm.style.display = "none";
                var params = {
                    Filename : dSMName + '_' + date + '.csv',
                    Contenttype : 'text/csv',
                    Data : me.exportCSV()
                };
                for(var par in params) {
                    var opt = document.createElement("textarea");
                    opt.name = par;
                    opt.value = params[par];
                    tempForm.appendChild(opt);
                }
                document.body.appendChild(tempForm);
                tempForm.submit();
                document.body.removeChild(tempForm);
            }
        });

        this.clearScreenButton = Ext.create('Ext.button.Button', {
            text : _("clear screen"),
            iconCls : 'icon-clear',
            handler : function() {
                me.lineChart.series.get(0).removeAllMarkers();
                me.meteringLineStore.removeAll();
                me.lineChart.redraw(false);
            }
        });
        this.clearScreenButton.disable();

        this.meteringLineStore = Ext.create('DSS.addon.Metering.MeteringStore');
        this.meteringLineStore.on('load', function() {
            me.lineChart.axes.get(0).maximum = me.calcYAxisMax(me.meteringLineStore.max('value'));
            me.lineChart.refresh();
        });

        //use two stores for the column chart. CompleteStore stores all metering
        // values, the second one contains only a subset of the first and is used
        // to display the values in the chart
        this.meteringColumnCompleteStore = Ext.create('DSS.addon.Metering.MeteringStore');
        this.meteringColumnStore = Ext.create('DSS.addon.Metering.MeteringStore');
        this.meteringColumnCompleteStore.on('load', function() {
            var count = me.meteringColumnCompleteStore.getCount();
            var records = me.meteringColumnCompleteStore.getRange(count - DSS.addon.Metering.columnValueCount, count - 1);
            me.meteringColumnStore.add(records);
        });
        this.meteringColumnStore.on('add', function() {
            me.columnChart.axes.get(1).fromDate = me.meteringColumnStore.min('date');
            me.columnChart.axes.get(1).toDate = me.meteringColumnStore.max('date');
            me.columnChart.axes.get(0).maximum = me.calcYAxisMax(me.meteringColumnStore.max('value'));
            me.columnChart.refresh();
        });

        this.meteringLineSeries = Ext.create('DSS.addon.Metering.LineSeries');
        this.meteringLineYAxis = Ext.create('DSS.addon.Metering.NumericAxis');
        this.meteringLineYAxis.title = _('instantaneous consumption') + ' (W)';
        this.meteringLineTimeAxis = Ext.create('DSS.addon.Metering.TimeAxis');

        this.lineChart = Ext.create('Ext.chart.Chart', {
            store : this.meteringLineStore,
            autoScroll : true,
            zoomX : 0,
            axes : [this.meteringLineYAxis, this.meteringLineTimeAxis],
            series : [this.meteringLineSeries],
            theme : 'MeteringTheme',

            originalFrom : null,
            originalToo : null,

            setZoom : function(zoomConfig) {
                var meToo = this, axes = meToo.axes, bbox = meToo.chartBBox, xScale = 1 / bbox.width, yScale = 1 / bbox.height, zoomer = {
                    x : zoomConfig.x * xScale,
                    y : zoomConfig.y * yScale,
                    width : zoomConfig.width * xScale,
                    height : zoomConfig.height * yScale
                };

                var axis = axes.get(1);
                if(!meToo.originalFrom) {
                    meToo.originalFrom = meToo.store.min('date');
                }
                if(!meToo.originalTo) {
                    meToo.originalTo = meToo.store.max('date');
                }
                var ends = axis.calcEnds();
                var from = (ends.to - ends.from) * zoomer.x + ends.from, to = (ends.to - ends.from) * zoomer.width + from;
                axis.minimum = from;
                axis.maximum = to;

                meToo.redraw(false);
            },
            restoreZoom : function(redraw) {
                var meToo = this;
                var axis = meToo.axes.get(1);
                if(meToo.originalFrom && meToo.originalTo) {
                    var ends = Ext.draw.Draw.snapEnds(meToo.orinalFrom, meToo.originalTo, meToo.majorTickSteps !== false ? (meToo.majorTickSteps + 1) : meToo.steps);

                    axis.minimum = ends.from;
                    axis.maximum = ends.to;

                    meToo.originalFrom = null;
                    meToo.orignalTo = null;
                }

                if(redraw) {
                    meToo.redraw(false);
                }
            },
            // override bindStore method so that the chart is not refreshed
            // automatically
            bindStore : function(store, initial) {
                var me = this;
                if(!initial && me.store) {
                    if(store !== me.store && me.store.autoDestroy) {
                        me.store.destroyStore();
                    } else {
                        me.store.un('datachanged', me.refresh, me);
                        me.store.un('add', me.delayRefresh, me);
                        me.store.un('remove', me.delayRefresh, me);
                        me.store.un('update', me.delayRefresh, me);
                        me.store.un('clear', me.refresh, me);
                    }
                }
                if(store) {
                    store = Ext.data.StoreManager.lookup(store);
                }
                me.store = store;
                if(store && !initial) {
                    me.refresh();
                }
            }
        });

        this.lineChart.on('refresh', function() {
            me.lineChart.series.get(0).updateMarkers();
        });

        this.meteringColumnSeries = Ext.create('DSS.addon.Metering.ColumnSeries');
        this.meteringColumnYAxis = Ext.create('DSS.addon.Metering.NumericAxis');
        this.meteringColumnYAxis.title = _('consumption') + ' (Wh)';
        this.meteringColumnTimeAxis = Ext.create('DSS.addon.Metering.TimeAxis');

        this.columnChart = Ext.create('Ext.chart.Chart', {
            store : this.meteringColumnStore,
            autoScroll : true,
            axes : [this.meteringColumnYAxis, this.meteringColumnTimeAxis],
            series : [this.meteringColumnSeries],
            theme : 'MeteringTheme',

            originalFrom : null,
            originalToo : null,

            setZoom : function(zoomConfig) {
                var meToo = this, axes = meToo.axes, bbox = meToo.chartBBox, xScale = 1 / bbox.width, yScale = 1 / bbox.height, zoomer = {
                    x : zoomConfig.x * xScale,
                    y : zoomConfig.y * yScale,
                    width : zoomConfig.width * xScale,
                    height : zoomConfig.height * yScale
                };

                var axis = axes.get(1);
                if(!meToo.originalFrom) {
                    meToo.originalFrom = meToo.store.min('date');
                }
                if(!meToo.originalTo) {
                    meToo.originalTo = meToo.store.max('date');
                }
                var ends = axis.calcEnds();
                var from = (ends.to - ends.from) * zoomer.x + ends.from, to = (ends.to - ends.from) * zoomer.width + from;
                axis.minimum = from;
                axis.maximum = to;

                meToo.redraw(false);
            },
            restoreZoom : function(redraw) {
                var meToo = this;
                var axis = meToo.axes.get(1);
                if(meToo.originalFrom && meToo.originalTo) {
                    var ends = Ext.draw.Draw.snapEnds(meToo.orinalFrom, meToo.originalTo, meToo.majorTickSteps !== false ? (meToo.majorTickSteps + 1) : meToo.steps);

                    axis.minimum = ends.from;
                    axis.maximum = ends.to;

                    meToo.originalFrom = null;
                    meToo.orignalTo = null;
                }

                if(redraw) {
                    meToo.redraw(false);
                }
            },
            // override bindStore method so that the chart is not refreshed
            // automatically
            bindStore : function(store, initial) {
                var me = this;
                if(!initial && me.store) {
                    if(store !== me.store && me.store.autoDestroy) {
                        me.store.destroyStore();
                    } else {
                        me.store.un('datachanged', me.refresh, me);
                        me.store.un('add', me.delayRefresh, me);
                        me.store.un('remove', me.delayRefresh, me);
                        me.store.un('update', me.delayRefresh, me);
                        me.store.un('clear', me.refresh, me);
                    }
                }
                if(store) {
                    store = Ext.data.StoreManager.lookup(store);
                }
                me.store = store;
                if(store && !initial) {
                    me.refresh();
                }
            }
        });

        this.interval = null;
        this.repaintLock = true;

        Ext.apply(this, {
            layout : 'fit',
            items : this.lineChart,
            tbar : [this.typeBox, this.timeBox, {
                xtype : 'tbseparator'
            }, this.zoomFieldContainer, '->', this.clearScreenButton, this.exportCSVButton],
            bbar : [this.panSlider],
            region : 'center',
            border : false,
            split : true,
            minWidth : 600
        });

        me.on('resize', function() {
            me.panSlider.setWidth(me.getWidth() - 5);
        });

        DSS.addon.Metering.CenterPanel.superclass.initComponent.apply(this, arguments);
    },
    initStores : function() {
        var i;
        for( i = 0; i < this.resolutionStore.getCount(); i++) {
            var record = this.resolutionStore.getAt(i);
            record.setName(DSS.addon.Metering.time[record.get('resolution')]);
        }
        this.typeBox.setValue('consumption');
        this.unit = 'W';

        this.resolutionStore.clearFilter();
        this.resolutionStore.filter({
            filterFn : function(item) {
                var resolution = item.get('resolution');
                if(resolution === 1 || resolution === 60) {
                    return true;
                }
                return false;
            }
        });
    },
    changeChartType : function() {
        if(this.typeBox.getValue() != this.typeBox.oldValue) {
            //chart type has to be changed
            if(this.typeBox.getValue() === 'consumption') {
                this.zoomSlider.reset();
                this.panSlider.reset();
            } else if(this.typeBox.getValue() === 'energy') {
                this.zoomSlider.reset();
                this.panSlider.setValue(100, false);
            }
            this.typeBox.oldValue = this.typeBox.getValue();
        }
        if(this.timeBox.getValue() != this.timeBox.oldValue) {
            if(this.typeBox.getValue() === 'energy') {
                this.panSlider.setValue(100, false);
            }
            this.timeBox.oldValue = this.timeBox.getValue();
        }
    },
    repaint : function() {
        // check if a repaint is already in progress. If so then return
        // directly, otherwise do the repaint
        if(this.repaintLock) {
            this.repaintLock = false;
            var me = this;
            var dsid = this.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedDSID();
            var url;
            var value;

            // stop running interval (not stopping it may lead to incorrect
            // values
            // added to the store)
            this.stopInterval();

            this.changeChartType();

            if(this.typeBox.getValue() === 'consumption') {
                //check if something is selected, otherwise select second entry
                // (first entry would enable realtime data)
                if(this.timeBox.getValue() === undefined) {
                    this.timeBox.setValue(this.resolutionStore.getAt(1).get('resolution'));
                }

                this.unit = 'W';

                //show only the specified values in the timeBox
                this.resolutionStore.clearFilter();
                this.resolutionStore.filter({
                    filterFn : function(item) {
                        var resolution = item.get('resolution');
                        if(resolution === 1 || resolution === 60) {
                            return true;
                        }
                        return false;
                    }
                });

                //only allow selected values that are in the store
                value = this.timeBox.getValue();
                if((value !== 1 & value !== 60)) {
                    this.timeBox.setValue(this.resolutionStore.getAt(1));
                }

                this.lineChart.axes.get(1).step = DSS.addon.Metering.lineSteps[this.timeBox.getValue()];
                this.lineChart.axes.get(1).minorTickSteps = DSS.addon.Metering.lineMinorTicks[this.timeBox.getValue()];
                this.lineChart.axes.get(1).setDateFormat(DSS.addon.Metering.dateFormat[this.timeBox.getValue()]);
                url = '/json/metering/getValues?dsid=' + dsid + '&type=consumption&resolution=' + this.timeBox.getValue() + '&valueCount=600';

                if(this.items.get(0) != this.lineChart) {
                    //show line chart, set column chart invisible due to
                    // performance reasons
                    this.columnChart.setVisible(false);
                    this.lineChart.setVisible(true);
                    this.remove(this.columnChart, false);
                    this.add(this.lineChart);
                }

                //elements might be disabled, because the column chart was
                // displayed
                me.zoomSlider.enable();
                me.decrementZoomButton.enable();
                me.incrementZoomButton.enable();

                this.lineChart.restoreZoom(false);

                this.meteringLineStore.removeAll();
                this.meteringLineStore.getProxy().url = url;
                this.meteringLineStore.load();

            } else {
                //check if something is selected, otherwise select first entry
                if(this.timeBox.getValue() === undefined) {
                    this.timeBox.setValue(this.resolutionStore.getAt(0).get('resolution'));
                }

                this.unit = 'Wh';

                //show only the specified values in the timeBox
                this.resolutionStore.clearFilter();
                this.resolutionStore.filter({
                    filterFn : function(item) {
                        var resolution = item.get('resolution');
                        if(resolution === 60 || resolution === 900 || resolution === 86400 || resolution === 604800) {
                            return true;
                        }
                        return false;
                    }
                });

                //only allow selected values that are in the store
                value = this.timeBox.getValue();
                if((value !== 60 & value !== 900 & value !== 86400 & value !== 604800)) {
                    this.timeBox.setValue(this.resolutionStore.getAt(0));
                }

                this.columnChart.axes.get(1).step = DSS.addon.Metering.columnSteps[this.timeBox.getValue()];
                this.columnChart.axes.get(1).minorTickSteps = DSS.addon.Metering.columnMinorTicks[this.timeBox.getValue()];
                this.columnChart.axes.get(1).setDateFormat(DSS.addon.Metering.dateFormat[this.timeBox.getValue()]);
                url = '/json/metering/getValues?dsid=' + dsid + '&type=energyDelta&resolution=' + this.timeBox.getValue();

                if(this.items.get(0) != this.columnChart) {
                    // show column chart, set line chart invisible due to
                    // performance reasons
                    this.lineChart.setVisible(false);
                    this.columnChart.setVisible(true);
                    this.remove(this.lineChart, false);
                    this.add(this.columnChart);
                }

                //no zooming for the column chart!
                me.zoomSlider.disable();
                me.decrementZoomButton.disable();
                me.incrementZoomButton.disable();
                me.panSlider.setValue(100, false);

                this.columnChart.restoreZoom(false);

                this.meteringColumnStore.removeAll();
                this.meteringColumnCompleteStore.removeAll();
                this.meteringColumnCompleteStore.getProxy().url = url;
                this.meteringColumnCompleteStore.load();
            }

            if(this.typeBox.getValue() === 'consumption' && this.timeBox.getValue() === 1) {
                this.stopInterval();
                this.startInterval();
            } else {
                this.stopInterval();
            }
            this.repaintLock = true;
        }
    },
    setChartZoom : function() {
        var me = this;

        var box = me.lineChart.chartBBox;

        //set the x-border of the zoom area, be aware of the current zoom level
        me.lineChart.zoomX = Math.min((box.width - box.width / (me.zoomSlider.getValue() / 100)) * me.panSlider.getValue() / 100, box.width - box.width / (me.zoomSlider.getValue() / 100));

        if(me.lineChart.isVisible()) {
            var lineBox = me.lineChart.chartBBox;
            if(lineBox) {
                var sel = {
                    x : me.lineChart.zoomX,
                    y : 0,
                    width : lineBox.width / (me.zoomSlider.getValue() / 100),
                    height : lineBox.height
                };
                me.lineChart.restoreZoom();
                me.lineChart.setZoom(sel);
            }
        }

        if(me.columnChart.isVisible()) {
            me.meteringColumnStore.removeAll();
            var count = me.meteringColumnCompleteStore.getCount();
            var rangeMin = Math.floor(me.panSlider.getValue() * (count -DSS.addon.Metering.columnValueCount)/ 100);
            var rangeMax = Math.floor(rangeMin + DSS.addon.Metering.columnValueCount);
            var records = me.meteringColumnCompleteStore.getRange(rangeMin, rangeMax);
            me.meteringColumnStore.add(records);
        }

        // no zooming for the column chart at the moment
        /*if(me.columnChart.isVisible()) {
         var columBox = me.columnChart.chartBBox;
         if(columBox) {
         var sel = {
         x : 0,
         y : 0,
         width : columBox.width / (me.zoomSlider.getValue() / 100),
         height : columBox.height
         };
         me.columnChart.restoreZoom();
         me.columnChart.setZoom(sel);
         }
         }*/
    },
    exportCSV : function() {
        var csvString, i = 0, r = null;
        if(this.typeBox.getValue() === 'consumption') {
            csvString = '"' + _("Time") + '";' + '"' + _("instantaneous consumption") + ' (' + _("in") + ' ' + this.unit + ')"' + '\n';
            for( i = 0; i < this.meteringLineStore.getCount(); i++) {
                r = this.meteringLineStore.getAt(i);
                csvString += '"' + Ext.Date.format(new Date(r.get('date')), 'd.m.Y G:i:s') + '";' + new String(r.get('value')).replace('.', ',') + '\n';
            }
        } else {
            csvString = '"' + _("Time") + '";' + '"' + _("consumption") + ' (' + _("in") + ' ' + this.unit + ')"' + '\n'
            for( i = 0; i < this.meteringColumnCompleteStore.getCount(); i++) {
                r = this.meteringColumnCompleteStore.getAt(i);
                csvString += '"' + Ext.Date.format(new Date(r.get('date')), 'd.m.Y G:i:s') + '";' + new String(r.get('value')).replace('.', ',') + '\n';
            }
        }
        return csvString;
    },
    // starts the polling of the new metering data every x seconds via a
    // setInterval method and an Ajax request
    // also jumps in the data series are detected and jump markers are added to
    // the right position
    startInterval : function() {
        var me = this;
        var i;

        //no zooming if realtime data is displayed
        me.zoomSlider.disable();
        me.decrementZoomButton.disable();
        me.incrementZoomButton.disable();
        me.panSlider.disable();
        me.clearScreenButton.enable();

        //detect jumps in the data of the store
        if(this.meteringLineStore.isLoading()) {
            this.meteringLineStore.on('load', function() {
                me.lineChart.axes.get(0).maximum = me.calcYAxisMax(me.meteringLineStore.max('value'));
                var count = me.meteringLineStore.getCount();
                for( i = 5; i < count; i++) {
                    var a = me.meteringLineStore.getAt(i - 5);
                    var b = me.meteringLineStore.getAt(i - 4);
                    var c = me.meteringLineStore.getAt(i - 2);
                    var d = me.meteringLineStore.getAt(i - 1);

                    me.detectJumps(a, b, c, d);
                }
            }, me, {
                single : true
            });
        } else {
            me.lineChart.axes.get(0).maximum = me.calcYAxisMax(me.meteringLineStore.max('value'));
            var count = me.meteringLineStore.getCount();
            for( i = 5; i < count; i++) {
                var a = me.meteringLineStore.getAt(i - 5);
                var b = me.meteringLineStore.getAt(i - 4);
                var c = me.meteringLineStore.getAt(i - 2);
                var d = me.meteringLineStore.getAt(i - 1);

                me.detectJumps(a, b, c, d);
            }
        }

        this.interval = setInterval(function() {
            //check if some information about a previous scene-call is missing.
            // If so then fetch the information again
            if(me.repaintCount != 0) {
                if(++me.repaintCounter == me.repaintCount) {
                    me.repaintCount = 0;
                    me.repaintCounter = 0;
                    Ext.Ajax.request({
                        url : '/json/property/getString?path=/scripts/metering/lastCalledScenes',
                        // set the timeout to the refresh interval since we will
                        // do
                        // another request every 2 seconds
                        timeout : DSS.addon.Metering.refreshInterval,
                        success : function(response) {
                            var jsonData = Ext.JSON.decode(response.responseText);
                            if(jsonData.ok) {
                                var jsonString = jsonData.result.value;
                                var lastCalledScenes = Ext.JSON.decode(jsonString);

                                var i;
                                var time;
                                for( i = lastCalledScenes.length - 1; i >= 0; i--) {
                                    var sceneNameFound = false;
                                    var eventNameFound = false;

                                    var zoneID = '';
                                    var groupID = '';
                                    var sceneID = '';
                                    var originDSID = '';
                                    var eventName = '';
                                    var sceneName = '';
                                    time = new Date(lastCalledScenes[i].time);
                                    zoneID = parseInt(lastCalledScenes[i].zoneID, 10);
                                    if(lastCalledScenes[i].groupID) {
                                        groupID = parseInt(lastCalledScenes[i].groupID, 10);
                                    }
                                    if(lastCalledScenes[i].originDSID) {
                                        originDSID = lastCalledScenes[i].originDSID;
                                    }
                                    if(lastCalledScenes[i].sceneID) {
                                        sceneID = parseInt(lastCalledScenes[i].sceneID, 10);
                                        sceneName = me.getSceneName(sceneID, groupID, zoneID);
                                    }
                                    if(lastCalledScenes[i].eventName) {
                                        var selectedDSID = me.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedDSID();
                                        var j;
                                        for( j = 0; j < me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getCount(); j++) {
                                            var meterDSID = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getAt(j).get('meterDSID');
                                            if(meterDSID === selectedDSID) {
                                                eventName = lastCalledScenes[i].eventName;
                                                eventNameFound = true;
                                                break;
                                            }
                                        }
                                    }
                                    if(sceneName !== undefined && sceneName !== '') {
                                        sceneNameFound = true;
                                    }

                                    if(sceneNameFound) {
                                        var sceneString;
                                        if(zoneID !== 0) {
                                            var zoneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).get('text');
                                            if(originDSID) {
                                                var originName = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getById(originDSID).get('text');
                                                //only 29 characters fit in a
                                                // bubble, shorten text if
                                                // necessary
                                                var length = sceneName.length + zoneName.length + originName.length + 7;
                                                if(length > 29) {
                                                    zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                                                }
                                                sceneString = originName + ' ' + _("in") + ' ' + zoneName + ' ' + sceneName;
                                            } else {
                                                //only 29 characters fit in a
                                                // bubble, shorten text if
                                                // necessary
                                                var length = sceneName.length + zoneName.length + 4;
                                                if(length > 29) {
                                                    zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                                                }
                                                sceneString = sceneName + ' ' + _("in") + ' ' + zoneName;
                                            }
                                        } else {
                                            sceneString = sceneName;
                                        }
                                        me.lineChart.series.get(0).updateMarkerText(time, '', sceneString);
                                    } else if(eventNameFound) {
                                        var zoneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).get('text');
                                        var originName = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getById(originDSID).get('text');

                                        var length = eventName.length + 8;
                                        if(length > 29) {
                                            eventName = eventName.substring(0, eventName.length - (length - 29) - 1) + '.';
                                        }
                                        var lineOne = ' - ' + eventName;
                                        length = originName.length + zoneName.length + 8;
                                        if(length > 29) {
                                            zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                                        }
                                        var lineTwo = _("for") + ' ' + originName + ' ' + _("in") + ' ' + zoneName;

                                        me.lineChart.series.get(0).updateMarkerText(time, lineOne, lineTwo);
                                    }
                                }
                            }
                        }
                    });
                }
            }

            Ext.Ajax.request({
                url : '/json/metering/getLatest?type=consumption&from=.meters(' + me.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedDSID() + ')',
                // set the timeout to the refresh interval since we will do
                // another request every 2 seconds
                timeout : DSS.addon.Metering.refreshInterval,
                success : function(response) {
                    var jsonData = Ext.JSON.decode(response.responseText);
                    if(jsonData.ok) {
                        var array = [];
                        //convert dateString to a format every browser
                        //should understand
                        var dateString = jsonData.result.values[0].date;
                        var date = Ext.Date.parse(dateString, 'Y-m-d g:i:s');
                        var record = [date, jsonData.result.values[0].value];
                        array.push(record);

                        if(me.meteringLineStore.getCount() >= 600) {
                            me.meteringLineStore.removeAt(0);
                        }
                        me.meteringLineStore.loadData(array, true);

                        // adjust Y-range so that no buble is cut off
                        me.lineChart.axes.get(0).maximum = Math.max(me.lineChart.axes.get(0).maximum, jsonData.result.values[0].value * 1.2);

                        var count = me.meteringLineStore.getCount();
                        var a = me.meteringLineStore.getAt(count - 7);
                        var b = me.meteringLineStore.getAt(count - 6);
                        var c = me.meteringLineStore.getAt(count - 4);
                        var d = me.meteringLineStore.getAt(count - 3);

                        me.detectJumps(a, b, c, d);

                        me.lineChart.refresh();
                    }
                }
            });
        }, DSS.addon.Metering.refreshInterval);
    },
    /**   detect jumps
     * the parameters of this function are storeItems
     * jump detection is based on the last 5 values
     *
     *            c----d
     *           /
     *          x
     *         /
     *   a----b
     *
     * a jump is detected for c, if
     *      |a-b| < 5
     *      |a-c| > 10
     *      |c-d| < 5
     *      |c-x| > 5
     *
     */
    detectJumps : function(itemA, itemB, itemC, itemD) {
        var me = this;

        if(!itemA || !itemB || !itemC || !itemD) {
            return;
        }

        var a = itemA.get('value');
        var b = itemB.get('value');
        var c = itemC.get('value');
        var d = itemD.get('value');

        if(Math.abs(a - b) < 5 && Math.abs(c - d) < 5 && Math.abs(a - c) > 10) {
            var diff = c - a;
            var diffString;
            if(diff > 0) {
                diffString = '+' + diff + ' ' + me.unit;
            } else {
                diffString = diff + ' ' + me.unit;
            }

            // load the last called scenes from the property tree
            Ext.Ajax.request({
                url : '/json/property/getString?path=/scripts/metering/lastCalledScenes',
                // set the timeout to the refresh interval since we will do
                // another request every 2 seconds
                timeout : DSS.addon.Metering.refreshInterval,
                success : function(response) {
                    var jsonData = Ext.JSON.decode(response.responseText);
                    if(jsonData.ok) {
                        var jsonString = jsonData.result.value;
                        var lastCalledScenes = Ext.JSON.decode(jsonString);

                        //set an interval of 10 seconds to compare the timestamp
                        // of the last called scene with
                        var now = new Date(itemD.get('date'));
                        var before = new Date(now);
                        before.setTime(before.getTime() - 10000);
                        var after = new Date(now);
                        after.setTime(after.getTime() + 10000);

                        var sceneNameFound = false;
                        var eventNameFound = false;

                        var zoneID;
                        var groupID;
                        var sceneID;
                        var originDSID;
                        var eventName;
                        var sceneName;

                        var i;
                        var time;
                        for( i = lastCalledScenes.length - 1; i >= 0; i--) {
                            time = new Date(lastCalledScenes[i].time);
                            if(before <= time && time <= after) {
                                zoneID = parseInt(lastCalledScenes[i].zoneID, 10);
                                if(lastCalledScenes[i].groupID) {
                                    groupID = parseInt(lastCalledScenes[i].groupID, 10);
                                }
                                if(lastCalledScenes[i].originDSID) {
                                    originDSID = lastCalledScenes[i].originDSID;
                                }
                                if(lastCalledScenes[i].sceneID) {
                                    sceneID = parseInt(lastCalledScenes[i].sceneID, 10);
                                    sceneName = me.getSceneName(sceneID, groupID, zoneID);
                                }
                                if(lastCalledScenes[i].eventName) {
                                    var selectedDSID = me.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedDSID();
                                    var j;
                                    for( j = 0; j < me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getCount(); j++) {
                                        var meterDSID = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getAt(j).get('meterDSID');
                                        if(meterDSID === selectedDSID) {
                                            eventName = lastCalledScenes[i].eventName;
                                            eventNameFound = true;
                                            break;
                                        }
                                    }
                                }
                                if(sceneName !== undefined && sceneName !== '') {
                                    sceneNameFound = true;
                                }
                                break;
                            }
                        }

                        if(sceneNameFound) {
                            var sceneString;
                            if(zoneID !== 0) {
                                var zoneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).get('text');
                                if(originDSID) {
                                    var originName = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getById(originDSID).get('text');
                                    //only 29 characters fit in a bubble, shorten
                                    // text if necessary
                                    var length = sceneName.length + zoneName.length + originName.length + 7;
                                    if(length > 29) {
                                        zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                                    }
                                    sceneString = originName + ' ' + _("in") + ' ' + zoneName + ' ' + sceneName;
                                } else {
                                    //only 29 characters fit in a bubble, shorten
                                    // text if necessary
                                    var length = sceneName.length + zoneName.length + 4;
                                    if(length > 29) {
                                        zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                                    }
                                    sceneString = sceneName + ' ' + _("in") + ' ' + zoneName;
                                }
                            } else {
                                sceneString = sceneName;
                            }
                            me.lineChart.series.get(0).addJumpMarker(itemD, diffString, sceneString, time);
                        } else if(eventNameFound) {
                            var zoneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).get('text');
                            var originName = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getById(originDSID).get('text');

                            var length = diff.length + eventName.length + 3;
                            if(length > 29) {
                                eventName = eventName.substring(0, eventName.length - (length - 29) - 1) + '.';
                            }
                            var lineOne = diffString + ' - ' + eventName;
                            length = originName.length + zoneName.length + 8;
                            if(length > 29) {
                                zoneName = zoneName.substring(0, zoneName.length - (length - 29) - 1) + '.';
                            }
                            var lineTwo = _("for") + ' ' + originName + ' ' + _("in") + ' ' + zoneName;

                            me.lineChart.series.get(0).addJumpMarker(itemD, lineOne, lineTwo, time);
                        } else {
                            me.lineChart.series.get(0).addJumpMarker(itemD, diffString, '', itemD.get('date'));
                            me.repaintCount = 4;
                        }
                    }
                }
            });
        }
    },
    stopInterval : function() {
        if(this.interval != null) {
            clearInterval(this.interval);
            this.lineChart.series.get(0).removeAllMarkers();
            this.lineChart.redraw(false);
            this.clearScreenButton.disable();
            this.panSlider.enable();
        }
    },
    getSceneName : function(sceneID, groupID, zoneID) {
        var me = this;
        var sceneName = '';

        //if the zone of the last scene call is 0 we don't have to check if the
        // zone is handled by the dsm
        if(zoneID === 0) {
            sceneName = DSS.addon.Metering.sceneNames[sceneID];
        } else if(!groupID || groupID === 0) {
            sceneName = DSS.addon.Metering.sceneNames[sceneID];
        } else {
            //check if the zone of the matching scene call is handled by the
            // selected dSM
            var selectedDSID = me.findParentByType('mainpanel').dSMPanel.dSMView.getSelectedDSID();

            var j;
            for( j = 0; j < me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getCount(); j++) {
                var meterDSID = me.apartmentStore.getAt(0).zones().getById(zoneID).devices().getAt(j).get('meterDSID');
                if(meterDSID === selectedDSID) {
                    if(me.sceneNameStore.getAt(0).zones().getById(zoneID).groups().getById(groupID).scenes().getById(sceneID)) {
                        sceneName = me.sceneNameStore.getAt(0).zones().getById(zoneID).groups().getById(groupID).scenes().getById(sceneID).get('name');
                    } else {
                        sceneName = DSS.addon.Metering.sceneNames[sceneID];
                    }
                    break;
                }
            }
        }

        return sceneName;
    },
    /**
     * calculate the maximum value to display to display on the y axis, ensure
     * that this value is dividable by 10
     */
    calcYAxisMax : function(value) {
        var max = Math.max(DSS.addon.Metering.YAxisMax, value * 1.2);
        var div = 1;
        while(max / (div * 10) > 1) {
            div *= 10;
        }
        max = (Math.ceil(max / div)) * div;
        return max;
    }
});

/*######################################
 TimeAxis
 ######################################

 Behaves like a normal TimeAxis, but overrides the function to calculate
 the steps.
 */

Ext.define('DSS.addon.Metering.TimeAxis', {
    type : 'Time',
    position : 'bottom',
    fields : 'date',
    dateFormat : 'd. M - G:i',
    dashSize : 6,
    grid : true,

    setDateFormat : function(dateFormat) {
        var me = this, label, f;
        label = me.label || {};
        label.renderer = function(v) {
            return Ext.Date.format(new Date(v), dateFormat);
        };
    },
    //override the function since it is not working if the label is
    // corrupt
    drawHorizontalLabels : function() {
        var me = this, labelConf = me.label, floor = Math.floor, max = Math.max, axes = me.chart.axes, position = me.position, inflections = me.inflections, ln = inflections.length, labels = me.labels, labelGroup = me.labelGroup, maxHeight = 0, ratio, gutterY = me.chart.maxGutter[1], ubbox, bbox, point, prevX, prevLabel, projectedWidth = 0, textLabel, attr, textRight, text, label, last, x, y, i, firstLabel;
        last = ln - 1;
        //get a reference to the first text label dimensions
        point = inflections[0];
        firstLabel = me.getOrCreateLabel(0, me.label.renderer(labels[0]));
        ratio = Math.floor(Math.abs(Math.sin(labelConf.rotate && (labelConf.rotate.degrees * Math.PI / 180) || 0)));

        for( i = 0; i < ln; i++) {
            point = inflections[i];
            // here is my change
            // skip label if label can't be rendered
            if(labels[i] === Infinity) {
                continue;
            }
            text = me.label.renderer(labels[i]);
            textLabel = me.getOrCreateLabel(i, text);
            bbox = textLabel._bbox;
            maxHeight = max(maxHeight, bbox.height + me.dashSize + me.label.padding);
            x = floor(point[0] - ( ratio ? bbox.height : bbox.width) / 2);
            if(me.chart.maxGutter[0] == 0) {
                if(i == 0 && axes.findIndex('position', 'left') == -1) {
                    x = point[0];
                } else if(i == last && axes.findIndex('position', 'right') == -1) {
                    x = point[0] - bbox.width;
                }
            }
            if(position == 'top') {
                y = point[1] - (me.dashSize * 2) - me.label.padding - (bbox.height / 2);
            } else {
                y = point[1] + (me.dashSize * 2) + me.label.padding + (bbox.height / 2);
            }

            textLabel.setAttributes({
                hidden : false,
                x : x,
                y : y
            }, true);

            // Skip label if there isn't available minimum space
            if(i != 0 && (me.intersect(textLabel, prevLabel) || me.intersect(textLabel, firstLabel))) {
                textLabel.hide(true);
                continue;
            }
            prevLabel = textLabel;
        }

        return maxHeight;
    },
    calcEnds : function() {
        var me = this, range, step = me.step;
        if(step) {
            range = me.getRange();
            range = me.snapEndsByDateAndStep(new Date(range.min), new Date(range.max), Ext.isNumber(step) ? [Date.MILLI, step] : step);
            range.step = (range.to - range.from) / range.steps;
            return range;
        } else {
            return me.callParent(arguments);
        }
    },
    //own snapEndsByDateAndStep since Sencha is not able to do simple
    // calculations
    snapEndsByDateAndStep : function(from, to, step, lockEnds) {

        var fromStat = [from.getFullYear(), from.getMonth(), from.getDate(),
                from.getHours(), from.getMinutes(), from.getSeconds(), from.getMilliseconds()],
            steps = 0, testFrom, testTo;
        if (lockEnds) {
            testFrom = from;
        } else {
            switch (step[0]) {
                case Ext.Date.MILLI:
                    testFrom = new Date(fromStat[0], fromStat[1], fromStat[2], fromStat[3],
                            fromStat[4], fromStat[5], Math.floor(fromStat[6] / step[1]) * step[1]);
                    break;
                case Ext.Date.SECOND:
                    testFrom = new Date(fromStat[0], fromStat[1], fromStat[2], fromStat[3],
                            fromStat[4], Math.floor(fromStat[5] / step[1]) * step[1], 0);
                    break;
                case Ext.Date.MINUTE:
                    testFrom = new Date(fromStat[0], fromStat[1], fromStat[2], fromStat[3],
                            Math.floor(fromStat[4] / step[1]) * step[1], 0, 0);
                    break;
                case Ext.Date.HOUR:
                    testFrom = new Date(fromStat[0], fromStat[1], fromStat[2],
                            Math.floor(fromStat[3] / step[1]) * step[1], 0, 0, 0);
                    break;
                case Ext.Date.DAY:
                    //introduce brackets in Math.floor!
                    testFrom = new Date(fromStat[0], fromStat[1],
                            Math.floor((fromStat[2] - 1) / step[1]) * step[1] + 1, 0, 0, 0, 0);
                    break;
                case Ext.Date.MONTH:
                    testFrom = new Date(fromStat[0], Math.floor(fromStat[1] / step[1]) * step[1], 1, 0, 0, 0, 0);
                    break;
                default: // Ext.Date.YEAR
                    testFrom = new Date(Math.floor(fromStat[0] / step[1]) * step[1], 0, 1, 0, 0, 0, 0);
                    break;
            }
        }

        testTo = testFrom;
        // TODO(zhangbei) : We can do it better somehow...
        while(testTo < to) {
            testTo = Ext.Date.add(testTo, step[0], step[1]);
            steps++;
        }

        if(lockEnds) {
            testTo = to;
        }
        return {
            from : +testFrom,
            to : +testTo,
            step : (testTo - testFrom) / steps,
            steps : steps
        };
    }
});

/*######################################
 NumericAxis
 ######################################

 Behaves like a normal NumericAxis, but overrides the function to calculate
 the steps. Steps are now dividable by 10/100/1000 depending on the maximum
 value of the axis
 */

Ext.define('DSS.addon.Metering.NumericAxis', {
    type : 'Numeric',
    position : 'left',
    fields : ['value'],
    decimals : 0,
    maximum : DSS.addon.Metering.YAxisMax,
    minimum : 0,
    grid : true,

    //override method to calculate the steps manually
    calcEnds : function() {
        var me = this, fields = me.fields, range = me.getRange(), min = range.min, max = range.max, outfrom, outto, out;
        out = Ext.draw.Draw.snapEnds(min, max, me.majorTickSteps !== false ? (me.majorTickSteps + 1) : me.steps);
        outfrom = out.from;
        outto = out.to;
        if(me.forceMinMax) {
            if(!isNaN(max)) {
                out.to = max;
            }
            if(!isNaN(min)) {
                out.from = min;
            }
        }
        if(!isNaN(me.maximum)) {
            //TODO(nico) users are responsible for their own minimum/maximum
            // values set.
            //Clipping should be added to remove lines in the chart which are
            // below the axis.
            out.to = me.maximum;
        }
        if(!isNaN(me.minimum)) {
            //TODO(nico) users are responsible for their own minimum/maximum
            // values set.
            //Clipping should be added to remove lines in the chart which are
            // below the axis.
            out.from = me.minimum;
        }

        //Adjust after adjusting minimum and maximum
        out.step = (out.to - out.from) / (outto - outfrom) * out.step;
        //here the changes begin
        var div = 10;
        while(max / (div * 10) > 1) {
            div *= 10;
        }
        div /= 10;
        out.step = Math.ceil(out.step / div) * div;
        //end changes

        if(me.adjustMaximumByMajorUnit) {
            out.to += out.step;
        }
        if(me.adjustMinimumByMajorUnit) {
            out.from -= out.step;
        }
        me.prevMin = min == max ? 0 : min;
        me.prevMax = max;
        return out;
    }
});

/*######################################
 MeteringLineSeries
 ######################################

 Behaves like a normal LineSeries, but adds some functions to draw markers
 to specific storeItems.
 */

Ext.define('DSS.addon.Metering.LineSeries', {

    type : 'Line',
    xField : 'date',
    yField : 'value',
    axis : ['left', 'bottom'],
    smooth : false,
    showMarkers : false,
    highlight : false,
    style : {
        fill : '#00523b',
        stroke : '#00523b',
        'stroke-width' : 3
    },
    selectionTolerance : 5,

    constructor : function(config) {
        this.callParent(arguments);
        var me = this;
        Ext.apply(me, config, {
            tips : {
                constrainPosition : true,
                trackMouse : true,
                width : 200,
                height : 35,
                renderer : function(storeItem, item) {
                    this.setTitle(storeItem.get('value').toFixed(0) + 'W ' + _('at') + ' ' + Ext.Date.format(new Date(storeItem.get('date')), 'd.m.Y - G:i:s'));
                }
            }
        });
    },
    // array to store all markers
    jumpMarkers : [],

    // add a new marker to the given store item. The text is displayed in the
    // chart
    addJumpMarker : function(storeItem, diff, sceneName, time) {
        var me = this, i = 0, ln = me.items.length;

        // check if a bubble was already painted within the last 12 seconds (12000 ms). If so
        // do nothing otherwise paint a new bubble
        if(me.jumpMarkers.length === 0 || time - me.jumpMarkers[me.jumpMarkers.length - 1].time > 12000) {
            var complete = true;

            if(diff === '' || sceneName === '') {
                complete = false;
            }

            for( i = 0; i < ln; i++) {
                if(storeItem.get('date') == me.items[i].value[0] && storeItem.get('value') == me.items[i].value[1]) {
                    var diffSprite = Ext.create('Ext.draw.Sprite', {
                        type : 'text',
                        myType : 'diff',
                        complete : complete,
                        time : time,
                        text : diff,
                        x : me.items[i].point[0] - 195,
                        y : me.items[i].point[1] - 46,
                        zIndex : 7000 + me.jumpMarkers.length * 10 + 5,
                        storeItem : storeItem
                    });

                    var frameSprite = Ext.create('Ext.draw.Sprite', {
                        type : 'path',
                        myType : 'frame',
                        complete : complete,
                        time : time,
                        path : "M 0 10 L 0 40 Q 0 50 10 50 L 170 50 L 200 60 L 180 40 L 180 10 Q 180 0 170 0 L 10 0 Q 0 0 0 10 Z",
                        zIndex : 7000 + me.jumpMarkers.length * 10,
                        stroke : '#00523b',
                        'stroke-width' : 3,
                        fill : '#baefd1',
                        opacity : 1,
                        storeItem : storeItem
                    });

                    frameSprite.setAttributes({
                        translate : {
                            x : me.items[i].point[0] - 200,
                            y : me.items[i].point[1] - 60
                        }
                    }, true);

                    var sceneSprite = Ext.create('Ext.draw.Sprite', {
                        type : 'text',
                        myType : 'scene',
                        complete : complete,
                        time : time,
                        text : sceneName,
                        x : me.items[i].point[0] - 195,
                        y : me.items[i].point[1] - 28,
                        zIndex : 7000 + me.jumpMarkers.length * 10 + 5,
                        storeItem : storeItem
                    });

                    me.chart.surface.add(frameSprite);
                    me.chart.surface.add(diffSprite);
                    me.chart.surface.add(sceneSprite);
                    frameSprite.show(true);
                    diffSprite.show(true);
                    sceneSprite.show(true);

                    me.jumpMarkers.push(frameSprite);
                    me.jumpMarkers.push(diffSprite);
                    me.jumpMarkers.push(sceneSprite);
                }
            }
        }
    },
    getPositionForStoreItem : function(storeItem) {
        var me = this;

        var i, ln = me.items.length;
        for( i = 0; i < ln; i++) {
            if(storeItem.get('date') == me.items[i].value[0] && storeItem.get('value') == me.items[i].value[1]) {
                return me.items[i].point;
            }
        }
    },
    // updates the text of a bubble. this is needed if some information were not
    // available at creation of the bubble
    updateMarkerText : function(time, diff, sceneName) {
        var me = this;
        var i;
        for( i = 0; i < me.jumpMarkers.length; i++) {
            var sprite = me.jumpMarkers[i];
            var before = Ext.Date.add(sprite.time, Ext.Date.Second, -10);
            var after = Ext.Date.add(sprite.time, Ext.Date.Second, 10);
            if(Ext.Date.between(time, before, after)) {
                if(!sprite.complete) {
                    if(sprite.myType === 'diff') {
                        if(diff !== '') {
                            var text = sprite.text + diff;
                            sprite.setAttributes({
                                text : text
                            });
                        }
                    } else if(sprite.myType === 'scene' && sprite.text == '') {
                        sprite.setAttributes({
                            text : sceneName
                        });
                    }
                }
            }
        }
    },
    // sets all markers to the new position of the item. Necessary if a new item
    // is added to the chart
    updateMarkers : function() {
        var me = this;

        var i;
        for( i = 0; i < me.jumpMarkers.length; i++) {
            var sprite = me.jumpMarkers[i];
            var point = me.getPositionForStoreItem(sprite.storeItem);
            if(point === undefined) {
                me.jumpMarkers.splice(i, 1);
                sprite.remove();
            } else {
                if(sprite.myType === 'diff') {
                    sprite.setAttributes({
                        x : point[0] - 195,
                        y : point[1] - 46
                    }, true);
                } else if(sprite.myType === 'scene') {
                    sprite.setAttributes({
                        x : point[0] - 195,
                        y : point[1] - 28
                    }, true);
                } else {
                    sprite.setAttributes({
                        translate : {
                            x : point[0] - 200,
                            y : point[1] - 60
                        }
                    }, true);
                }
            }
        }
    },
    removeAllMarkers : function() {
        var me = this;
        var i;
        for( i = 0; i < me.jumpMarkers.length; i++) {
            me.jumpMarkers[i].remove();
        }
        me.jumpMarkers = [];
    },
    drawSeries : function() {
        var me = this, chart = me.chart, chartAxes = chart.axes, store = chart.getChartStore(), storeCount = store.getCount(), surface = me.chart.surface, bbox = {}, group = me.group, showMarkers = me.showMarkers, markerGroup = me.markerGroup, enableShadows = chart.shadow, shadowGroups = me.shadowGroups, shadowAttributes = me.shadowAttributes, smooth = me.smooth, lnsh = shadowGroups.length, dummyPath = ["M"], path = ["M"], renderPath = ["M"], smoothPath = ["M"], markerIndex = chart.markerIndex, axes = [].concat(me.axis), shadowBarAttr, xValues = [], xValueMap = {}, yValues = [], yValueMap = {}, onbreak = false, storeIndices = [], markerStyle = me.markerStyle, seriesStyle = me.style, colorArrayStyle = me.colorArrayStyle, colorArrayLength = colorArrayStyle && colorArrayStyle.length || 0, isNumber = Ext.isNumber, seriesIdx = me.seriesIdx, boundAxes = me.getAxesForXAndYFields(), boundXAxis = boundAxes.xAxis, boundYAxis = boundAxes.yAxis, shadows, shadow, shindex, fromPath, fill, fillPath, rendererAttributes, x, y, prevX, prevY, firstX, firstY, markerCount, i, j, ln, axis, ends, marker, markerAux, item, xValue, yValue, coords, xScale, yScale, minX, maxX, minY, maxY, line, animation, endMarkerStyle, endLineStyle, type, count, items;

        if(me.fireEvent('beforedraw', me) === false) {
            return;
        }

        //if store is empty or the series is excluded in the legend then there's
        // nothing to draw.
        if(!storeCount || me.seriesIsHidden) {
            items = this.items;
            if(items) {
                for( i = 0, ln = items.length; i < ln; ++i) {
                    if(items[i].sprite) {
                        items[i].sprite.hide(true);
                    }
                }
            }
            return;
        }

        //prepare style objects for line and markers
        endMarkerStyle = Ext.apply(markerStyle || {}, me.markerConfig);
        type = endMarkerStyle.type;
        delete endMarkerStyle.type;
        endLineStyle = seriesStyle;
        //if no stroke with is specified force it to 0.5 because this is
        //about making *lines*
        if(!endLineStyle['stroke-width']) {
            endLineStyle['stroke-width'] = 0.5;
        }
        //If we're using a time axis and we need to translate the points,
        //then reuse the first markers as the last markers.
        if(markerIndex && markerGroup && markerGroup.getCount()) {
            for( i = 0; i < markerIndex; i++) {
                marker = markerGroup.getAt(i);
                markerGroup.remove(marker);
                markerGroup.add(marker);
                markerAux = markerGroup.getAt(markerGroup.getCount() - 2);
                marker.setAttributes({
                    x : 0,
                    y : 0,
                    translate : {
                        x : markerAux.attr.translation.x,
                        y : markerAux.attr.translation.y
                    }
                }, true);
            }
        }

        me.unHighlightItem();
        me.cleanHighlights();

        me.setBBox();
        bbox = me.bbox;
        me.clipRect = [bbox.x, bbox.y, bbox.width, bbox.height];
        for( i = 0, ln = axes.length; i < ln; i++) {
            axis = chartAxes.get(axes[i]);
            if(axis) {
                ends = axis.calcEnds();
                if(axis.position == 'top' || axis.position == 'bottom') {
                    minX = ends.from;
                    maxX = ends.to;
                } else {
                    minY = ends.from;
                    maxY = ends.to;
                }
            }
        }
        // If a field was specified without a corresponding axis, create one to
        // get bounds
        //only do this for the axis where real values are bound (that's why we
        // check for
        //me.axis)
        if(me.xField && !isNumber(minX) && (boundXAxis == 'bottom' || boundXAxis == 'top') && !chartAxes.get(boundXAxis)) {
            axis = Ext.create('Ext.chart.axis.Axis', {
                chart : chart,
                fields : [].concat(me.xField)
            }).calcEnds();
            minX = axis.from;
            maxX = axis.to;
        }
        if(me.yField && !isNumber(minY) && (boundYAxis == 'right' || boundYAxis == 'left') && !chartAxes.get(boundYAxis)) {
            axis = Ext.create('Ext.chart.axis.Axis', {
                chart : chart,
                fields : [].concat(me.yField)
            }).calcEnds();
            minY = axis.from;
            maxY = axis.to;
        }
        if(isNaN(minX)) {
            minX = 0;
            xScale = bbox.width / ((storeCount - 1) || 1);
        } else {
            xScale = bbox.width / ((maxX - minX) || (storeCount - 1) || 1);
        }

        if(isNaN(minY)) {
            minY = 0;
            yScale = bbox.height / ((storeCount - 1) || 1);
        } else {
            yScale = bbox.height / ((maxY - minY) || (storeCount - 1) || 1);
        }

        // Extract all x and y values from the store
        me.eachRecord(function(record, i) {
            xValue = record.get(me.xField);

            // Ensure a value
            if( typeof xValue == 'string' || typeof xValue == 'object' && !Ext.isDate(xValue)
            //set as uniform distribution if the axis is a category axis.
            || boundXAxis && chartAxes.get(boundXAxis) && chartAxes.get(boundXAxis).type == 'Category') {
                if( xValue in xValueMap) {
                    xValue = xValueMap[xValue];
                } else {
                    xValue = xValueMap[xValue] = i;
                }
            }

            // Filter out values that don't fit within the pan/zoom area
            if(xValue < minX || xValue > maxX) {
                return;
            }

            // Filter out values that don't fit within the pan/zoom buffer area
            yValue = record.get(me.yField);
            //skip undefined values
            if( typeof yValue == 'undefined' || ( typeof yValue == 'string' && !yValue)) {
                //<debug warn>
                if(Ext.isDefined(Ext.global.console)) {
                    Ext.global.console.warn("[Ext.chart.series.Line]  Skipping a store element with an undefined value at ", record, xValue, yValue);
                }
                //</debug>
                return;
            }
            // Ensure a value
            if( typeof yValue == 'string' || typeof yValue == 'object' && !Ext.isDate(yValue)
            //set as uniform distribution if the axis is a category axis.
            || boundYAxis && chartAxes.get(boundYAxis) && chartAxes.get(boundYAxis).type == 'Category') {
                yValue = i;
            }
            storeIndices.push(i);
            xValues.push(xValue);
            yValues.push(yValue);
        });
        ln = xValues.length;
        if(ln > bbox.width) {
            coords = me.shrink(xValues, yValues, bbox.width);
            xValues = coords.x;
            yValues = coords.y;
        }

        me.items = [];
        count = 0;
        ln = xValues.length;
        for( i = 0; i < ln; i++) {
            xValue = xValues[i];
            yValue = yValues[i];
            if(yValue === false) {
                if(path.length == 1) {
                    path = [];
                }
                onbreak = true;
                me.items.push(false);
                continue;
            } else {
                x = (bbox.x + (xValue - minX) * xScale).toFixed(2);
                y = ((bbox.y + bbox.height) - (yValue - minY) * yScale).toFixed(2);
                if(onbreak) {
                    onbreak = false;
                    path.push('M');
                }
                path = path.concat([x, y]);
            }
            if(( typeof firstY == 'undefined') && ( typeof y != 'undefined')) {
                firstY = y;
                firstX = x;
            }
            // If this is the first line, create a dummypath to animate in from.
            if(!me.line || chart.resizing) {
                dummyPath = dummyPath.concat([x, bbox.y + bbox.height / 2]);
            }

            // When resizing, reset before animating
            if(chart.animate && chart.resizing && me.line) {
                me.line.setAttributes({
                    path : dummyPath
                }, true);
                if(me.fillPath) {
                    me.fillPath.setAttributes({
                        path : dummyPath,
                        opacity : 0.2
                    }, true);
                }
                if(me.line.shadows) {
                    shadows = me.line.shadows;
                    for( j = 0, lnsh = shadows.length; j < lnsh; j++) {
                        shadow = shadows[j];
                        shadow.setAttributes({
                            path : dummyPath
                        }, true);
                    }
                }
            }
            if(showMarkers) {
                marker = markerGroup.getAt(count++);
                if(!marker) {
                    marker = Ext.chart.Shape[type](surface, Ext.apply({
                        group : [group, markerGroup],
                        x : 0,
                        y : 0,
                        translate : {
                            x : +(prevX || x),
                            y : prevY || (bbox.y + bbox.height / 2)
                        },
                        value : '"' + xValue + ', ' + yValue + '"',
                        zIndex : 4000
                    }, endMarkerStyle));
                    marker._to = {
                        translate : {
                            x : +x,
                            y : +y
                        }
                    };
                } else {
                    marker.setAttributes({
                        value : '"' + xValue + ', ' + yValue + '"',
                        x : 0,
                        y : 0,
                        hidden : false
                    }, true);
                    marker._to = {
                        translate : {
                            x : +x,
                            y : +y
                        }
                    };
                }
            }
            me.items.push({
                series : me,
                value : [xValue, yValue],
                point : [x, y],
                sprite : marker,
                storeItem : store.getAt(storeIndices[i])
            });
            prevX = x;
            prevY = y;
        }

        if(path.length <= 1) {
            //nothing to be rendered
            return;
        }

        if(me.smooth) {
            smoothPath = Ext.draw.Draw.smooth(path, isNumber(smooth) ? smooth : me.defaultSmoothness);
        }
        renderPath = smooth ? smoothPath : path;

        //Correct path if we're animating timeAxis intervals
        if(chart.markerIndex && me.previousPath) {
            fromPath = me.previousPath;
            if(!smooth) {
                Ext.Array.erase(fromPath, 1, 2);
            }
        } else {
            fromPath = path;
        }

        // Only create a line if one doesn't exist.
        if(!me.line) {
            me.line = surface.add(Ext.apply({
                type : 'path',
                group : group,
                path : dummyPath,
                stroke : endLineStyle.stroke || endLineStyle.fill
            }, endLineStyle || {}));

            if(enableShadows) {
                me.line.setAttributes(Ext.apply({}, me.shadowOptions), true);
            }

            //unset fill here (there's always a default fill withing the themes).
            me.line.setAttributes({
                fill : 'none',
                zIndex : 3000
            });
            if(!endLineStyle.stroke && colorArrayLength) {
                me.line.setAttributes({
                    stroke : colorArrayStyle[seriesIdx % colorArrayLength]
                }, true);
            }
            if(enableShadows) {
                //create shadows
                shadows = me.line.shadows = [];
                for( shindex = 0; shindex < lnsh; shindex++) {
                    shadowBarAttr = shadowAttributes[shindex];
                    shadowBarAttr = Ext.apply({}, shadowBarAttr, {
                        path : dummyPath
                    });
                    shadow = surface.add(Ext.apply({}, {
                        type : 'path',
                        group : shadowGroups[shindex]
                    }, shadowBarAttr));
                    shadows.push(shadow);
                }
            }
        }
        if(me.fill) {
            fillPath = renderPath.concat([["L", x, bbox.y + bbox.height], ["L", firstX, bbox.y + bbox.height], ["L", firstX, firstY]]);
            if(!me.fillPath) {
                me.fillPath = surface.add({
                    group : group,
                    type : 'path',
                    opacity : endLineStyle.opacity || 0.3,
                    fill : endLineStyle.fill || colorArrayStyle[seriesIdx % colorArrayLength],
                    path : dummyPath
                });
            }
        }
        markerCount = showMarkers && markerGroup.getCount();
        if(chart.animate) {
            fill = me.fill;
            line = me.line;
            //Add renderer to line. There is not unique record associated with
            // this.
            rendererAttributes = me.renderer(line, false, {
                path : renderPath
            }, i, store);
            Ext.apply(rendererAttributes, endLineStyle || {}, {
                stroke : endLineStyle.stroke || endLineStyle.fill
            });
            //fill should not be used here but when drawing the special fill path
            // object
            delete rendererAttributes.fill;
            line.show(true);
            if(chart.markerIndex && me.previousPath) {
                me.animation = animation = me.onAnimate(line, {
                    to : rendererAttributes,
                    from : {
                        path : fromPath
                    }
                });
            } else {
                me.animation = animation = me.onAnimate(line, {
                    to : rendererAttributes
                });
            }
            //animate shadows
            if(enableShadows) {
                shadows = line.shadows;
                for( j = 0; j < lnsh; j++) {
                    shadows[j].show(true);
                    if(chart.markerIndex && me.previousPath) {
                        me.onAnimate(shadows[j], {
                            to : {
                                path : renderPath
                            },
                            from : {
                                path : fromPath
                            }
                        });
                    } else {
                        me.onAnimate(shadows[j], {
                            to : {
                                path : renderPath
                            }
                        });
                    }
                }
            }
            //animate fill path
            if(fill) {
                me.fillPath.show(true);
                me.onAnimate(me.fillPath, {
                    to : Ext.apply({}, {
                        path : fillPath,
                        fill : endLineStyle.fill || colorArrayStyle[seriesIdx % colorArrayLength],
                        'stroke-width' : 0
                    }, endLineStyle || {})
                });
            }
            //animate markers
            if(showMarkers) {
                count = 0;
                for( i = 0; i < ln; i++) {
                    if(me.items[i]) {
                        item = markerGroup.getAt(count++);
                        if(item) {
                            rendererAttributes = me.renderer(item, store.getAt(i), item._to, i, store);
                            me.onAnimate(item, {
                                to : Ext.apply(rendererAttributes, endMarkerStyle || {})
                            });
                            item.show(true);
                        }
                    }
                }
                for(; count < markerCount; count++) {
                    item = markerGroup.getAt(count);
                    item.hide(true);
                }
                //                for(i = 0; i < (chart.markerIndex || 0)-1; i++)
                // {
                //                    item = markerGroup.getAt(i);
                //                    item.hide(true);
                //                }
            }
        } else {
            rendererAttributes = me.renderer(me.line, false, {
                path : renderPath,
                hidden : false
            }, i, store);
            Ext.apply(rendererAttributes, endLineStyle || {}, {
                stroke : endLineStyle.stroke || endLineStyle.fill
            });
            //fill should not be used here but when drawing the special fill path
            // object
            delete rendererAttributes.fill;
            me.line.setAttributes(rendererAttributes, true);
            //set path for shadows
            if(enableShadows) {
                shadows = me.line.shadows;
                for( j = 0; j < lnsh; j++) {
                    shadows[j].setAttributes({
                        path : renderPath,
                        hidden : false
                    }, true);
                }
            }
            if(me.fill) {
                me.fillPath.setAttributes({
                    path : fillPath,
                    hidden : false
                }, true);
            }
            if(showMarkers) {
                count = 0;
                for( i = 0; i < ln; i++) {
                    if(me.items[i]) {
                        item = markerGroup.getAt(count++);
                        if(item) {
                            rendererAttributes = me.renderer(item, store.getAt(i), item._to, i, store);
                            item.setAttributes(Ext.apply(endMarkerStyle || {}, rendererAttributes || {}), true);
                            item.show(true);
                        }
                    }
                }
                for(; count < markerCount; count++) {
                    item = markerGroup.getAt(count);
                    item.hide(true);
                }
            }
        }

        if(chart.markerIndex) {
            if(me.smooth) {
                Ext.Array.erase(path, 1, 2);
            } else {
                Ext.Array.splice(path, 1, 0, path[1], path[2]);
            }
            me.previousPath = path;
        }
        me.renderLabels();
        me.renderCallouts();

        me.fireEvent('draw', me);
    },
    isItemInPoint : function(x, y, item, i) {

        var me = this,
            items = me.items,
            tolerance = me.selectionTolerance,
            result = null,
            prevItem,
            nextItem,
            prevPoint,
            nextPoint,
            ln,
            x1,
            y1,
            x2,
            y2,
            xIntersect,
            yIntersect,
            dist1, dist2, dist, midx, midy,
            sqrt = Math.sqrt, abs = Math.abs;

        nextItem = items[i];
        prevItem = i && items[i - 1];

        if(i >= ln) {
            prevItem = items[ln - 1];
        }
        prevPoint = prevItem && prevItem.point;
        nextPoint = nextItem && nextItem.point;
        x1 = prevItem ? prevPoint[0] : nextPoint[0] - tolerance / 10;
        y1 = prevItem ? prevPoint[1] : nextPoint[1];
        x2 = nextItem ? nextPoint[0] : prevPoint[0] + tolerance * 10;
        y2 = nextItem ? nextPoint[1] : prevPoint[1];
        dist1 = sqrt((x - x1) * (x - x1) + (y - y1) * (y - y1));
        dist2 = sqrt((x - x2) * (x - x2) + (y - y2) * (y - y2));
        dist = Math.min(dist1, dist2);

        if(dist <= tolerance) {
            return dist == dist1 ? prevItem : nextItem;
        }
        return false;
    }
});

/*######################################
 ColumnSeries
 ######################################
 The definition of the ColumnSeries for the Wh-graph. The getPath method had to
 be overridden, due to the incorrect calculation of the x-coordinate of the bar
 */

Ext.define('DSS.addon.Metering.ColumnSeries', {
    type : 'column',
    xField : 'date',
    yField : 'value',
    tips : {
        constrainPosition : true,
        trackMouse : true,
        width : 200,
        height : 35,
        renderer : function(storeItem, item) {
            this.setTitle(storeItem.get('value').toFixed(0) + 'Wh ' + _('at') + ' ' + Ext.Date.format(new Date(storeItem.get('date')), 'd.m.Y - G:i:s'));
        }
    },
    getPaths : function() {
        var me = this, chart = me.chart, store = chart.getChartStore(), bounds = me.bounds = me.getBounds(), items = me.items = [], gutter = me.gutter / 100, groupGutter = me.groupGutter / 100, animate = chart.animate, column = me.column, group = me.group, enableShadows = chart.shadow, shadowGroups = me.shadowGroups, shadowAttributes = me.shadowAttributes, shadowGroupsLn = shadowGroups.length, bbox = bounds.bbox, xPadding = me.xPadding, yPadding = me.yPadding, stacked = me.stacked, barsLen = bounds.barsLen, colors = me.colorArrayStyle, colorLength = colors && colors.length || 0, math = Math, mmax = math.max, mmin = math.min, mabs = math.abs, j, yValue, height, totalDim, totalNegDim, bottom, top, hasShadow, barAttr, attrs, counter, shadowIndex, shadow, sprite, offset, floorY;

        // here the changes begin:
        // first get the timeAxis and calculate the start and end
        // dates,
        // then calculate the start-coordinate and the width of the
        // chart
        var timeAxis = me.chart.axes.get(1);
        var ends = timeAxis.calcEnds();
        var endsRange = ends.to - ends.from;
        var xMin = bbox.x + xPadding;
        var xWidth = (store.getCount() * bounds.barWidth * (1 + gutter));

        store.each(function(record, i, total) {
            bottom = bounds.zero;
            top = bounds.zero;
            totalDim = 0;
            totalNegDim = 0;
            hasShadow = false;

            for( j = 0, counter = 0; j < barsLen; j++) {
                // Excluded series
                if(me.__excludes && me.__excludes[j]) {
                    continue;
                }
                yValue = record.get(bounds.bars[j]);
                height = Math.round((yValue - mmax(bounds.minY, 0)) * bounds.scale);
                barAttr = {
                    fill : colors[(barsLen > 1 ? j : 0) % colorLength]
                };
                if(column) {
                    Ext.apply(barAttr, {
                        height : height,
                        width : mmax(bounds.groupBarWidth, 0),
                        // set the x-coordinate of the bar according
                        // to the date of the record
                        x : (xWidth / endsRange) * (record.get('date') - ends.from) + xMin,
                        y : bottom - height
                    });
                } else {
                    // draw in reverse order
                    offset = (total - 1) - i;
                    Ext.apply(barAttr, {
                        height : mmax(bounds.groupBarWidth, 0),
                        width : height + (bottom == bounds.zero),
                        x : bottom + (bottom != bounds.zero),
                        y : (bbox.y + yPadding + offset * bounds.barWidth * (1 + gutter) + counter * bounds.groupBarWidth * (1 + groupGutter) * !stacked + 1)
                    });
                }
                if(height < 0) {
                    if(column) {
                        barAttr.y = top;
                        barAttr.height = mabs(height);
                    } else {
                        barAttr.x = top + height;
                        barAttr.width = mabs(height);
                    }
                }
                if(stacked) {
                    if(height < 0) {
                        top += height * ( column ? -1 : 1);
                    } else {
                        bottom += height * ( column ? -1 : 1);
                    }
                    totalDim += mabs(height);
                    if(height < 0) {
                        totalNegDim += mabs(height);
                    }
                }
                barAttr.x = Math.floor(barAttr.x) + 1;
                floorY = Math.floor(barAttr.y);
                if(!Ext.isIE9 && barAttr.y > floorY) {
                    floorY--;
                }
                barAttr.y = floorY;
                barAttr.width = Math.floor(barAttr.width);
                barAttr.height = Math.floor(barAttr.height);
                items.push({
                    series : me,
                    storeItem : record,
                    value : [record.get(me.xField), yValue],
                    attr : barAttr,
                    point : column ? [barAttr.x + barAttr.width / 2, yValue >= 0 ? barAttr.y : barAttr.y + barAttr.height] : [yValue >= 0 ? barAttr.x + barAttr.width : barAttr.x, barAttr.y + barAttr.height / 2]
                });
                // When resizing, reset before animating
                if(animate && chart.resizing) {
                    attrs = column ? {
                        x : barAttr.x,
                        y : bounds.zero,
                        width : barAttr.width,
                        height : 0
                    } : {
                        x : bounds.zero,
                        y : barAttr.y,
                        width : 0,
                        height : barAttr.height
                    };
                    if(enableShadows && (stacked && !hasShadow || !stacked)) {
                        hasShadow = true;
                        //update shadows
                        for( shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
                            shadow = shadowGroups[shadowIndex].getAt( stacked ? i : (i * barsLen + j));
                            if(shadow) {
                                shadow.setAttributes(attrs, true);
                            }
                        }
                    }
                    //update sprite position and width/height
                    sprite = group.getAt(i * barsLen + j);
                    if(sprite) {
                        sprite.setAttributes(attrs, true);
                    }
                }
                counter++;
            }
            if(stacked && items.length) {
                items[i * counter].totalDim = totalDim;
                items[i * counter].totalNegDim = totalNegDim;
            }
        }, me);
    }
});

/*######################################
 MeteringModel
 ######################################*/

Ext.define('DSS.addon.Metering.MeteringModel', {
    extend : 'Ext.data.Model',
    fields : [{
        name : 'date',
        type : 'date',
        dateFormat : 'timestamp',
        mapping : 0
    }, {
        name : 'value',
        mapping : 1
    }]
});

/*######################################
 MeteringStore
 ######################################*/

Ext.define('DSS.addon.Metering.MeteringStore', {
    extend : 'Ext.data.Store',

    model : 'DSS.addon.Metering.MeteringModel',
    proxy : {
        type : 'ajax',
        actionMethods : {
            read : 'GET'
        },
        pageParam : undefined,
        limitParam : undefined,
        startParam : undefined,
        reader : {
            type : 'array',
            root : 'result.values',
            successProperty : 'ok'
        }
    },
    sorters : [{
        property : 'date',
        direction : 'ASC'
    }],
    // override remove method of this store so that no 'datachanged' or 'remove'
    // event is fired
    // -> chart is only redrawn when a new value pair is added to the store
    // -> better performance
    remove : function(records, /* private */isMove) {
        if(!Ext.isArray(records)) {
            records = [records];
        }
        /*
         * Pass the isMove parameter if we know we're going to be re-inserting
         * this record
         */
        isMove = isMove === true;
        var me = this, sync = false, i = 0, length = records.length, isPhantom, index, record;

        for(i; i < length; i++) {
            record = records[i];
            index = me.data.indexOf(record);

            if(me.snapshot) {
                me.snapshot.remove(record);
            }
            if(index > -1) {
                isPhantom = record.phantom === true;
                if(!isMove && !isPhantom) {
                    // don't push phantom records onto removed
                    me.removed.push(record);
                }
                record.unjoin(me);
                me.data.remove(record);
                sync = sync || !isPhantom;
            }
        }

        if(!isMove && me.autoSync && sync) {
            me.sync();
        }
    }
});

/*######################################
 MeteringTheme
 ######################################*/

Ext.define('Ext.chart.theme.MeteringTheme', {
    extend : 'Ext.chart.theme.Base',

    constructor : function(config) {
        var baseColor = '#005138';
        var fill = '#00523b';

        this.callParent([Ext.apply({
            axis : {
                fill : baseColor,
                stroke : baseColor
            },
            axisLabelLeft : {
                fill : baseColor
            },
            axisLabelBottom : {
                fill : baseColor
            },
            axisTitleLeft : {
                fill : baseColor
            },
            axisTitleBottom : {
                fill : baseColor
            },
            colors : [fill],
            markerThemes : [{
                fill : baseColor,
                type : 'circle'
            }]
        }, config)]);
    }
});

Ext.apply(Ext.Date, {
    monthNames : [
        _("January"),
        _("February"),
        _("March"),
        _("April"),
        _("May"),
        _("June"),
        _("July"),
        _("August"),
        _("September"),
        _("October"),
        _("November"),
        _("December")
    ]
});
